/******************************************************************************
 *  FileName:       IS01BFRGB_SPI.c
 *  Dependencies:
 *  Processor:      ATMEGA88
 *  Compiler:       WinAVR
 *
 * SOFTWARE IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
 * WARRENTY OF ANY KIND, WHETHER EXPRESS OF IMPLIED, INCLUDING BUT NOT
 * LIMITED TO, THE IMPLIED WARRENTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
 * BE LIABLE FOR A INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES, LOST PROFITS, OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
 * PROCUREMENT OF SUBSTITUTE GOOD, TECHNOLOGY OF SERVICES, AND CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * ANY CLAIMS FOR INDEMNITY OF CONTRIBUTION, OR OTHER SIMILAR COSTS.
 * DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OF COPYRIGHT
 * INFRINGEMENT.
 *
 * Version History
 * Version 1.0 05/01/2009 Brandon Tougas
 * Initial Public Release
 ******************************************************************************/

#include <avr/io.h>
#include <util/delay.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/fuse.h>

void SPI_MasterInit(void);
void SPI_MasterTransmit(unsigned char);
void Initialize_PORTS(void);
void IS01BFRGB_SPI(void);
void Initialize_INTERRUPTS(void);
void Initialize_TIMER0(void);
void Initialize_TIMER1(void);

volatile signed char c_Row;
volatile signed char c_Column;
volatile signed char c_Bit;

volatile signed char c_Bitmap = 1;
volatile unsigned char uc_Data;

extern unsigned char IS01BFRGB_bitmap_1[24][5] PROGMEM;
extern unsigned char IS01BFRGB_bitmap_2[24][5] PROGMEM;
extern unsigned char IS01BFRGB_bitmap_3[24][5] PROGMEM;
extern unsigned char IS01BFRGB_bitmap_4[24][5] PROGMEM;

// Serial clock pin of the IS01BFRGB
#define IS01BFRGB_SCP   PORTB5
#define SCP_PORT        PORTB

// Data In pin of the IS01BFRGB
#define IS01BFRGB_DIN   PORTB3
#define DIN_PORT        PORTB

// Latch Pulse pin of the IS01BFRGB
#define IS01BFRGB_LP    PORTB0
#define LP_PORT         PORTB

// First Line Marker pin of the IS01BFRGB
#define IS01BFRGB_FLM   PORTD7
#define FLM_PORT        PORTD

#define SWITCH          PORTD2
#define S_PORT          PIND

#define DDR_SPI         DDRB
#define DD_SCK          DDB5
#define DD_MISO         DDB4
#define DD_MOSI         DDB3
#define DD_SS           DDB2

/************************** PWM Duty Cycle constants **************************/

#define FULL            0xFF //0x00
#define THREE_QUARTER   0xC0 //0x40
#define HALF            0x80
#define ONE_QUARTER     0x40 //0xC0
#define OFF             0x00 //0xFF
/******************************************************************************/

#define MAX_BITMAP      6       // Indicates the number of bitmaps used

#define     HIGH    1
#define     LOW     0

/******************************************************************************
 *
 *      Function Name:  Fuses
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: To set the devices fuses within the code
 *
 ******************************************************************************/

FUSES =
{
    .low = (FUSE_CKSEL3 & FUSE_CKSEL2 & FUSE_CKSEL0 
           & FUSE_SUT1 & FUSE_SUT0 & FUSE_CKDIV8),

    .high = (FUSE_SPIEN & FUSE_BODLEVEL2 & FUSE_BODLEVEL0)
};

ISR(PCINT2_vect, ISR_BLOCK)
{
    PCMSK2 &= ~(1 << PCINT18);      // Disable PA7 for a Pin Change Interrupt

    /* Do the interrupt routine after the button has been released */
    if ((!(S_PORT & (1 << SWITCH)) == LOW))
    {
        c_Bitmap++;

        if (c_Bitmap >= MAX_BITMAP)
        {
            c_Bitmap = 1;
        }
    }

    PCMSK2 |= (1 << PCINT18);      // Enable PA7 for a Pin Change Interrupt
}

/******************************************************************************
 *
 *      Function Name:  Initialize_INTERRUPTS
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose:   Configure the pin change interrupt on PA7 and enable
 *                 Global interrupts.
 *
 ******************************************************************************/

void Initialize_INTERRUPTS(void)
{
    cli();      // Clear global interrupts

    PCICR |= (1 << PCIE2);      // Enable Pin Change Interrupt 2, On PORTD

    PCIFR |= (1 << PCIF2);      // Clear Pin Change Interrupt Flag 2,

    PCMSK2 |= (1 << PCINT18);   // Enable PD2 & PD3 for a Pin Change Interrupt

    sei();      // Enable global interrupts
}

/******************************************************************************
 *
 *      Function Name:  Initialize_TIMER0
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: Timer 0 is used as two PWM outputs for two of the LEDs
 *               backlights. The brightness of the LEDs are controlled by
 *               the duty cycle of the PWMs. The duty cycles of the PWMs are
 *               controlled by the OCR1xH and OCR1xL registers.
 *
 ******************************************************************************/

void Initialize_TIMER0(void)
{
    TCCR0A |= ((1 << COM0A1) | (1 << COM0A0) | (1 << COM0B1) | (0 << COM0B0)
               | (1 << WGM01) | (1 << WGM00));

    TCCR0B |= ((0 << FOC0A) | (0 << FOC0B) | (0 << WGM02) | (0 << CS02)
               | (0 << CS01) | (1 << CS00));

    TCNT0 = 0x00;

    OCR0A = ONE_QUARTER;

    OCR0B = HALF;

    TIMSK1 |= ((0 << ICIE1) | (0 << OCIE1B) | (0 << OCIE1A) | (0 << TOIE1));
}

/******************************************************************************
 *
 *      Function Name:  Initialize_TIMER1
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: Timer 1 is used as two PWM outputs for one of the LEDs
 *               backlights. The brightness of the LEDs are controlled by
 *               the duty cycle of the PWMs. The duty cycles of the PWMs are
 *               controlled by the OCR1xH and OCR1xL registers.
 *
 ******************************************************************************/

void Initialize_TIMER1(void)
{
    TCCR1A |= ((1 << COM1A1) | (1 << COM1A0) | (0 << COM1B1) | (0 << COM1B0)
               | (0 << WGM11) | (1 << WGM10));

    TCCR1B |= ((0 << ICNC1) | (0 << ICES1) | (0 << WGM13) | (1 << WGM12)
               | (0 << CS12) | (0 << CS11) | (1 << CS10));

    TCNT1H = 0x00;
    TCNT1L = 0x00;

    OCR1AH = 0x00;
    OCR1AL = THREE_QUARTER;

    TIMSK1 |= ((0 << ICIE1) | (0 << OCIE1B) | (0 << OCIE1A) | (0 << TOIE1));
}

/******************************************************************************
 *
 *      Function Name:  SPI_Initialization
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose: Initialize the SPI as a Master.
 *               Set MOSI, SCK and SS to outputs.
 *               Enable SPI, Set Data Order to LSB first.
 *               Clock rate to fck/16, Clock Polarity leading edge as Falling
 *               and Clock Phase to sampleing on leading edge. See data sheet
 *               for more details.
 *
 ******************************************************************************/

void SPI_MasterInit(void)
{
    /* Set MOSI and SCK output, all others input */
    DDR_SPI |= ((1 << DD_MOSI) | (1 << DD_SCK) | (1 << DD_SS));

    /* Enable SPI, Master, set clock rate fck/16 */
    SPSR |= (0 << SPI2X);

    SPCR |= ((0 << SPIE) | (1 << SPE)  | (1 << DORD) | (1 << MSTR)
             | (1 << CPOL) | (0 << CPHA) | (0 << SPR1) | (1 << SPR0));
}

/******************************************************************************
 *
 *      Function Name:  SPI_MasterTransmit
 *
 *      Returns: None
 *
 *      Parameters: uc_Data
 *
 *      Purpose: Transmits data over SPI
 **
 *****************************************************************************/

void SPI_MasterTransmit(unsigned char uc_Data)
{
    /* Start transmission */
    SPDR = uc_Data;

    /* Wait for transmission to complete */
    while (!(SPSR & (1<<SPIF)));
}

/******************************************************************************
*
*      Function Name: Initialize_PORTS
*
*      Returns: None
*
*      Parameters: None
*
*      Purpose: Initialize Ports B, C, and D
*
*******************************************************************************/

void Initialize_PORTS(void)
{
    /***************************** PORT A ***********************************/
    /***************************** PORT B ***********************************/

    DDRB |= ((1 << DDB0) | (1 << DDB1) | (1 << DDB2) | (0 << DDB3)
             | (0 << DDB4) | (0 << DDB5) | (0 << DDB6) | (0 << DDB7));

    PORTB |= ((0 << PORTB0) | (0 << PORTB1) | (1 << PORTB2) | (0 << PORTB3)
              | (0 << PORTB4) | (0 << PORTB5) | (0 << PORTB6) | (0 << PORTB7));

    /***************************** PORT C ***********************************/

    DDRC |= ((0 << DDC0) | (0 << DDC1) | (0 << DDC2) | (0 << DDC3)
             | (0 << DDC4) | (0 << DDC5) | (0 << DDC6));

    PORTC |= ((0 << PORTC0) | (0 << PORTC1) | (0 << PORTC2) | (0 << PORTC3)
              | (0 << PORTC4) | (0 << PORTC5) | (0 << PORTC6));

    /***************************** PORT D ***********************************/

    DDRD |= ((0 << DDD0) | (0 << DDD1) | (0 << DDD2) | (0 << DDD3)
             | (0 << DDD4) | (1 << DDD5) | (1 << DDD6) | (1 << DDD7));

    PORTD |= ((0 << PORTD0) | (0 << PORTD1) | (0 << PORTD2) | (0 << PORTD3)
              | (0 << PORTD4) | (0 << PORTD5) | (0 << PORTD6) | (0 << PORTD7));

    /***************************** PORT E ***********************************/
}

/******************************************************************************
 *
 *      Function Name: IS01BFRGB_SPI
 *
 *      Returns: None
 *
 *      Parameters: None
 *
 *      Purpose:  To refresh the display using the SPI peripheral
 *
 ******************************************************************************/

void IS01BFRGB_SPI(void)
{
    while (1)   // Loop forever
    {
        /* Start at Row 0 of the Bitmap and increament to Row 23 */
        for (c_Row = 0; c_Row <= 23; c_Row++)
        {
            /* Start at Column 4 and decreament to column 0 */
            for (c_Column = 4; c_Column >= 0; c_Column--)
            {
//                _delay_us(200);

                /* Indicate the first line of the display by setting FML high */
                if ((c_Row == 0) && (c_Column == 0))
                {
                    FLM_PORT |= (1 << IS01BFRGB_FLM);
                }

                switch (c_Bitmap) // determines which Bitmap to use
                {
                case 1:
                    /* Retrieve bitmap data and send it to the display */
                    SPI_MasterTransmit(pgm_read_byte(
                                       &(IS01BFRGB_bitmap_1[c_Row][c_Column])));
                    break;

                case 2:
                    /* Retrieve bitmap data
                       Invert data and send it to the display */
                    SPI_MasterTransmit(~(pgm_read_byte(
                                      &(IS01BFRGB_bitmap_1[c_Row][c_Column]))));
                    break;

                case 3:
                    /* Retrieve bitmap data, send it to the display */
                    SPI_MasterTransmit(pgm_read_byte(
                                       &(IS01BFRGB_bitmap_2[c_Row][c_Column])));
                    break;

                case 4:
                    /* Retrieve bitmap data, send it to the display */
                    SPI_MasterTransmit(pgm_read_byte(
                                       &(IS01BFRGB_bitmap_3[c_Row][c_Column])));
                    break;

                case 5:
                    /* Retrieve bitmap data, send it to the display */
                    SPI_MasterTransmit(pgm_read_byte(
                                       &(IS01BFRGB_bitmap_4[c_Row][c_Column])));
                    break;

                default:
                    /* Retrieve bitmap data and send it to the display */
                    SPI_MasterTransmit(pgm_read_byte(
                                       &(IS01BFRGB_bitmap_1[c_Row][c_Column])));
                    break;
                }

                if (c_Column == 0)
                {
                    // Toggle LP pin of the display
                    LP_PORT |= (1 << IS01BFRGB_LP);
                    LP_PORT &= ~(1 << IS01BFRGB_LP);
                    FLM_PORT &= ~(1 << IS01BFRGB_FLM);
                }
            }
        }
    }
}

/******************************************************************************
 *
 *      Function Name: main
 *
 *      Returns: Integer
 *
 *      Parameters: None
 *
 *      Purpose: Main function
 *
 ******************************************************************************/

int main(void)
{

    Initialize_PORTS();

    SPI_MasterInit();

    Initialize_TIMER0();

    Initialize_TIMER1();

    Initialize_INTERRUPTS();

    IS01BFRGB_SPI();

    return(0);
}
